# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .key_get_response import KeyGetResponse as KeyGetResponse
from .ping_get_response import PingGetResponse as PingGetResponse
from .search_get_params import SearchGetParams as SearchGetParams
from .coin_get_id_params import CoinGetIDParams as CoinGetIDParams
from .exchange_get_params import ExchangeGetParams as ExchangeGetParams
from .global_get_response import GlobalGetResponse as GlobalGetResponse
from .nft_get_id_response import NFTGetIDResponse as NFTGetIDResponse
from .nft_get_list_params import NFTGetListParams as NFTGetListParams
from .search_get_response import SearchGetResponse as SearchGetResponse
from .coin_get_id_response import CoinGetIDResponse as CoinGetIDResponse
from .exchange_get_response import ExchangeGetResponse as ExchangeGetResponse
from .nft_get_list_response import NFTGetListResponse as NFTGetListResponse
from .exchange_get_id_params import ExchangeGetIDParams as ExchangeGetIDParams
from .nft_get_markets_params import NFTGetMarketsParams as NFTGetMarketsParams
from .derivative_get_response import DerivativeGetResponse as DerivativeGetResponse
from .exchange_get_id_response import ExchangeGetIDResponse as ExchangeGetIDResponse
from .exchange_get_list_params import ExchangeGetListParams as ExchangeGetListParams
from .nft_get_markets_response import NFTGetMarketsResponse as NFTGetMarketsResponse
from .asset_platform_get_params import AssetPlatformGetParams as AssetPlatformGetParams
from .exchange_get_list_response import ExchangeGetListResponse as ExchangeGetListResponse
from .exchange_rate_get_response import ExchangeRateGetResponse as ExchangeRateGetResponse
from .asset_platform_get_response import AssetPlatformGetResponse as AssetPlatformGetResponse
from .token_list_get_all_json_response import TokenListGetAllJsonResponse as TokenListGetAllJsonResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["MarketGetParams"]


class MarketGetParams(TypedDict, total=False):
    vs_currency: Required[str]
    """
    target currency of coins and market data \\**refers to
    [`/simple/supported_vs_currencies`](/reference/simple-supported-currencies).
    """

    category: str
    """
    filter based on coins' category \\**refers to
    [`/coins/categories/list`](/reference/coins-categories-list).
    """

    ids: str
    """coins' IDs, comma-separated if querying more than 1 coin.

    \\**refers to [`/coins/list`](/reference/coins-list).
    """

    include_tokens: Literal["top", "all"]
    """
    for `symbols` lookups, specify `all` to include all matching tokens Default
    `top` returns top-ranked tokens (by market cap or volume)
    """

    locale: Literal[
        "ar",
        "bg",
        "cs",
        "da",
        "de",
        "el",
        "en",
        "es",
        "fi",
        "fr",
        "he",
        "hi",
        "hr",
        "hu",
        "id",
        "it",
        "ja",
        "ko",
        "lt",
        "nl",
        "no",
        "pl",
        "pt",
        "ro",
        "ru",
        "sk",
        "sl",
        "sv",
        "th",
        "tr",
        "uk",
        "vi",
        "zh",
        "zh-tw",
    ]
    """language background, default: en"""

    names: str
    """coins' names, comma-separated if querying more than 1 coin."""

    order: Literal["market_cap_asc", "market_cap_desc", "volume_asc", "volume_desc", "id_asc", "id_desc"]
    """sort result by field, default: market_cap_desc"""

    page: float
    """page through results, default: 1"""

    per_page: float
    """total results per page, default: 100 Valid values: 1...250"""

    precision: Literal[
        "full", "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", "16", "17", "18"
    ]
    """decimal place for currency price value"""

    price_change_percentage: str
    """
    include price change percentage timeframe, comma-separated if query more than 1
    price change percentage timeframe Valid values: 1h, 24h, 7d, 14d, 30d, 200d, 1y
    """

    sparkline: bool
    """include sparkline 7 days data, default: false"""

    symbols: str
    """coins' symbols, comma-separated if querying more than 1 coin."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .token_price_get_addresses_params import TokenPriceGetAddressesParams as TokenPriceGetAddressesParams
from .token_price_get_addresses_response import TokenPriceGetAddressesResponse as TokenPriceGetAddressesResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["NetworkGetResponse", "Data", "DataAttributes"]


class DataAttributes(BaseModel):
    coingecko_asset_platform_id: Optional[str] = None

    name: Optional[str] = None


class Data(BaseModel):
    id: Optional[str] = None

    attributes: Optional[DataAttributes] = None

    type: Optional[str] = None


class NetworkGetResponse(BaseModel):
    data: Optional[List[Data]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = [
    "ContractGetContractAddressResponse",
    "Ath",
    "AthChangePercentage",
    "AthDate",
    "BannerImage",
    "Explorer",
    "FloorPrice",
    "FloorPrice14dPercentageChange",
    "FloorPrice1yPercentageChange",
    "FloorPrice24hPercentageChange",
    "FloorPrice30dPercentageChange",
    "FloorPrice60dPercentageChange",
    "FloorPrice7dPercentageChange",
    "Image",
    "Links",
    "MarketCap",
    "MarketCap24hPercentageChange",
    "Volume24h",
    "Volume24hPercentageChange",
]


class Ath(BaseModel):
    native_currency: Optional[float] = None

    usd: Optional[float] = None


class AthChangePercentage(BaseModel):
    native_currency: Optional[float] = None

    usd: Optional[float] = None


class AthDate(BaseModel):
    native_currency: Optional[datetime] = None

    usd: Optional[datetime] = None


class BannerImage(BaseModel):
    small: Optional[str] = None


class Explorer(BaseModel):
    link: Optional[str] = None

    name: Optional[str] = None


class FloorPrice(BaseModel):
    native_currency: Optional[float] = None

    usd: Optional[float] = None


class FloorPrice14dPercentageChange(BaseModel):
    native_currency: Optional[float] = None

    usd: Optional[float] = None


class FloorPrice1yPercentageChange(BaseModel):
    native_currency: Optional[float] = None

    usd: Optional[float] = None


class FloorPrice24hPercentageChange(BaseModel):
    native_currency: Optional[float] = None

    usd: Optional[float] = None


class FloorPrice30dPercentageChange(BaseModel):
    native_currency: Optional[float] = None

    usd: Optional[float] = None


class FloorPrice60dPercentageChange(BaseModel):
    native_currency: Optional[float] = None

    usd: Optional[float] = None


class FloorPrice7dPercentageChange(BaseModel):
    native_currency: Optional[float] = None

    usd: Optional[float] = None


class Image(BaseModel):
    small: Optional[str] = None

    small_2x: Optional[str] = None


class Links(BaseModel):
    discord: Optional[str] = None

    homepage: Optional[str] = None

    twitter: Optional[str] = None


class MarketCap(BaseModel):
    native_currency: Optional[float] = None

    usd: Optional[float] = None


class MarketCap24hPercentageChange(BaseModel):
    native_currency: Optional[float] = None

    usd: Optional[float] = None


class Volume24h(BaseModel):
    native_currency: Optional[float] = None

    usd: Optional[float] = None


class Volume24hPercentageChange(BaseModel):
    native_currency: Optional[float] = None

    usd: Optional[float] = None


class ContractGetContractAddressResponse(BaseModel):
    id: Optional[str] = None
    """NFT collection ID"""

    asset_platform_id: Optional[str] = None
    """NFT collection asset platform ID"""

    ath: Optional[Ath] = None
    """NFT collection all time highs"""

    ath_change_percentage: Optional[AthChangePercentage] = None
    """NFT collection all time highs change percentage"""

    ath_date: Optional[AthDate] = None
    """NFT collection all time highs date"""

    banner_image: Optional[BannerImage] = None
    """NFT collection banner image url"""

    contract_address: Optional[str] = None
    """NFT collection contract address"""

    description: Optional[str] = None
    """NFT collection description"""

    explorers: Optional[List[Explorer]] = None
    """NFT collection block explorers links"""

    floor_price: Optional[FloorPrice] = None
    """NFT collection floor price"""

    floor_price_14d_percentage_change: Optional[FloorPrice14dPercentageChange] = None
    """NFT collection floor price 14 days percentage change"""

    floor_price_1y_percentage_change: Optional[FloorPrice1yPercentageChange] = None
    """NFT collection floor price 1 year percentage change"""

    floor_price_24h_percentage_change: Optional[FloorPrice24hPercentageChange] = None

    floor_price_30d_percentage_change: Optional[FloorPrice30dPercentageChange] = None
    """NFT collection floor price 30 days percentage change"""

    floor_price_60d_percentage_change: Optional[FloorPrice60dPercentageChange] = None
    """NFT collection floor price 60 days percentage change"""

    floor_price_7d_percentage_change: Optional[FloorPrice7dPercentageChange] = None
    """NFT collection floor price 7 days percentage change"""

    floor_price_in_usd_24h_percentage_change: Optional[float] = None
    """NFT collection floor price in usd 24 hours percentage change"""

    image: Optional[Image] = None
    """NFT collection image url"""

    links: Optional[Links] = None
    """NFT collection links"""

    market_cap: Optional[MarketCap] = None
    """NFT collection market cap"""

    market_cap_24h_percentage_change: Optional[MarketCap24hPercentageChange] = None
    """NFT collection market cap 24 hours percentage change"""

    market_cap_rank: Optional[float] = None
    """coin market cap rank"""

    name: Optional[str] = None
    """NFT collection name"""

    native_currency: Optional[str] = None
    """NFT collection native currency"""

    native_currency_symbol: Optional[str] = None
    """NFT collection native currency symbol"""

    number_of_unique_addresses: Optional[float] = None
    """number of unique address owning the NFTs"""

    number_of_unique_addresses_24h_percentage_change: Optional[float] = None
    """number of unique address owning the NFTs 24 hours percentage change"""

    one_day_average_sale_price: Optional[float] = None
    """NFT collection one day average sale price"""

    one_day_average_sale_price_24h_percentage_change: Optional[float] = None
    """NFT collection one day average sale price 24 hours percentage change"""

    one_day_sales: Optional[float] = None
    """NFT collection one day sales"""

    one_day_sales_24h_percentage_change: Optional[float] = None
    """NFT collection one day sales 24 hours percentage change"""

    symbol: Optional[str] = None
    """NFT collection symbol"""

    total_supply: Optional[float] = None
    """NFT collection total supply"""

    user_favorites_count: Optional[float] = None
    """NFT collection user favorites count"""

    volume_24h: Optional[Volume24h] = None
    """NFT collection volume in 24 hours"""

    volume_24h_percentage_change: Optional[Volume24hPercentageChange] = None
    """NFT collection volume in 24 hours percentage change"""

    volume_in_usd_24h_percentage_change: Optional[float] = None
    """NFT collection volume in usd 24 hours percentage change"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .exchanges import (
    ExchangesResource,
    AsyncExchangesResource,
    ExchangesResourceWithRawResponse,
    AsyncExchangesResourceWithRawResponse,
    ExchangesResourceWithStreamingResponse,
    AsyncExchangesResourceWithStreamingResponse,
)
from .derivatives import (
    DerivativesResource,
    AsyncDerivativesResource,
    DerivativesResourceWithRawResponse,
    AsyncDerivativesResourceWithRawResponse,
    DerivativesResourceWithStreamingResponse,
    AsyncDerivativesResourceWithStreamingResponse,
)

__all__ = [
    "ExchangesResource",
    "AsyncExchangesResource",
    "ExchangesResourceWithRawResponse",
    "AsyncExchangesResourceWithRawResponse",
    "ExchangesResourceWithStreamingResponse",
    "AsyncExchangesResourceWithStreamingResponse",
    "DerivativesResource",
    "AsyncDerivativesResource",
    "DerivativesResourceWithRawResponse",
    "AsyncDerivativesResourceWithRawResponse",
    "DerivativesResourceWithStreamingResponse",
    "AsyncDerivativesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from coingecko_sdk import Coingecko, AsyncCoingecko
from coingecko_sdk.types.onchain.networks import (
    TrendingPoolGetResponse,
    TrendingPoolGetNetworkResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTrendingPools:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_get(self, client: Coingecko) -> None:
        trending_pool = client.onchain.networks.trending_pools.get()
        assert_matches_type(TrendingPoolGetResponse, trending_pool, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_get_with_all_params(self, client: Coingecko) -> None:
        trending_pool = client.onchain.networks.trending_pools.get(
            duration="5m",
            include="include",
            page=0,
        )
        assert_matches_type(TrendingPoolGetResponse, trending_pool, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_get(self, client: Coingecko) -> None:
        response = client.onchain.networks.trending_pools.with_raw_response.get()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        trending_pool = response.parse()
        assert_matches_type(TrendingPoolGetResponse, trending_pool, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_get(self, client: Coingecko) -> None:
        with client.onchain.networks.trending_pools.with_streaming_response.get() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            trending_pool = response.parse()
            assert_matches_type(TrendingPoolGetResponse, trending_pool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_get_network(self, client: Coingecko) -> None:
        trending_pool = client.onchain.networks.trending_pools.get_network(
            network="eth",
        )
        assert_matches_type(TrendingPoolGetNetworkResponse, trending_pool, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_get_network_with_all_params(self, client: Coingecko) -> None:
        trending_pool = client.onchain.networks.trending_pools.get_network(
            network="eth",
            duration="5m",
            include="include",
            page=0,
        )
        assert_matches_type(TrendingPoolGetNetworkResponse, trending_pool, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_get_network(self, client: Coingecko) -> None:
        response = client.onchain.networks.trending_pools.with_raw_response.get_network(
            network="eth",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        trending_pool = response.parse()
        assert_matches_type(TrendingPoolGetNetworkResponse, trending_pool, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_get_network(self, client: Coingecko) -> None:
        with client.onchain.networks.trending_pools.with_streaming_response.get_network(
            network="eth",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            trending_pool = response.parse()
            assert_matches_type(TrendingPoolGetNetworkResponse, trending_pool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_get_network(self, client: Coingecko) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `network` but received ''"):
            client.onchain.networks.trending_pools.with_raw_response.get_network(
                network="",
            )


class TestAsyncTrendingPools:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_get(self, async_client: AsyncCoingecko) -> None:
        trending_pool = await async_client.onchain.networks.trending_pools.get()
        assert_matches_type(TrendingPoolGetResponse, trending_pool, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_with_all_params(self, async_client: AsyncCoingecko) -> None:
        trending_pool = await async_client.onchain.networks.trending_pools.get(
            duration="5m",
            include="include",
            page=0,
        )
        assert_matches_type(TrendingPoolGetResponse, trending_pool, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_get(self, async_client: AsyncCoingecko) -> None:
        response = await async_client.onchain.networks.trending_pools.with_raw_response.get()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        trending_pool = await response.parse()
        assert_matches_type(TrendingPoolGetResponse, trending_pool, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_get(self, async_client: AsyncCoingecko) -> None:
        async with async_client.onchain.networks.trending_pools.with_streaming_response.get() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            trending_pool = await response.parse()
            assert_matches_type(TrendingPoolGetResponse, trending_pool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_network(self, async_client: AsyncCoingecko) -> None:
        trending_pool = await async_client.onchain.networks.trending_pools.get_network(
            network="eth",
        )
        assert_matches_type(TrendingPoolGetNetworkResponse, trending_pool, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_network_with_all_params(self, async_client: AsyncCoingecko) -> None:
        trending_pool = await async_client.onchain.networks.trending_pools.get_network(
            network="eth",
            duration="5m",
            include="include",
            page=0,
        )
        assert_matches_type(TrendingPoolGetNetworkResponse, trending_pool, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_get_network(self, async_client: AsyncCoingecko) -> None:
        response = await async_client.onchain.networks.trending_pools.with_raw_response.get_network(
            network="eth",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        trending_pool = await response.parse()
        assert_matches_type(TrendingPoolGetNetworkResponse, trending_pool, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_get_network(self, async_client: AsyncCoingecko) -> None:
        async with async_client.onchain.networks.trending_pools.with_streaming_response.get_network(
            network="eth",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            trending_pool = await response.parse()
            assert_matches_type(TrendingPoolGetNetworkResponse, trending_pool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_get_network(self, async_client: AsyncCoingecko) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `network` but received ''"):
            await async_client.onchain.networks.trending_pools.with_raw_response.get_network(
                network="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.exchanges import ticker_get_params
from ...types.exchanges.ticker_get_response import TickerGetResponse

__all__ = ["TickersResource", "AsyncTickersResource"]


class TickersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TickersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return TickersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TickersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return TickersResourceWithStreamingResponse(self)

    def get(
        self,
        id: str,
        *,
        coin_ids: str | NotGiven = NOT_GIVEN,
        depth: bool | NotGiven = NOT_GIVEN,
        dex_pair_format: Literal["contract_address", "symbol"] | NotGiven = NOT_GIVEN,
        include_exchange_logo: bool | NotGiven = NOT_GIVEN,
        order: Literal["trust_score_desc", "trust_score_asc", "volume_desc", "volume_asc", "base_target"]
        | NotGiven = NOT_GIVEN,
        page: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TickerGetResponse:
        """
        This endpoint allows you to **query exchange's tickers based on exchange’s ID**

        Args:
          coin_ids: filter tickers by coin IDs, comma-separated if querying more than 1 coin
              \\**refers to [`/coins/list`](/reference/coins-list).

          depth: include 2% orderbook depth (Example: cost_to_move_up_usd &
              cost_to_move_down_usd),default: false

          dex_pair_format:
              set to `symbol` to display DEX pair base and target as symbols, default:
              `contract_address`

          include_exchange_logo: include exchange logo, default: false

          order: use this to sort the order of responses, default: trust_score_desc

          page: page through results

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/exchanges/{id}/tickers",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "coin_ids": coin_ids,
                        "depth": depth,
                        "dex_pair_format": dex_pair_format,
                        "include_exchange_logo": include_exchange_logo,
                        "order": order,
                        "page": page,
                    },
                    ticker_get_params.TickerGetParams,
                ),
            ),
            cast_to=TickerGetResponse,
        )


class AsyncTickersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTickersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTickersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTickersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncTickersResourceWithStreamingResponse(self)

    async def get(
        self,
        id: str,
        *,
        coin_ids: str | NotGiven = NOT_GIVEN,
        depth: bool | NotGiven = NOT_GIVEN,
        dex_pair_format: Literal["contract_address", "symbol"] | NotGiven = NOT_GIVEN,
        include_exchange_logo: bool | NotGiven = NOT_GIVEN,
        order: Literal["trust_score_desc", "trust_score_asc", "volume_desc", "volume_asc", "base_target"]
        | NotGiven = NOT_GIVEN,
        page: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TickerGetResponse:
        """
        This endpoint allows you to **query exchange's tickers based on exchange’s ID**

        Args:
          coin_ids: filter tickers by coin IDs, comma-separated if querying more than 1 coin
              \\**refers to [`/coins/list`](/reference/coins-list).

          depth: include 2% orderbook depth (Example: cost_to_move_up_usd &
              cost_to_move_down_usd),default: false

          dex_pair_format:
              set to `symbol` to display DEX pair base and target as symbols, default:
              `contract_address`

          include_exchange_logo: include exchange logo, default: false

          order: use this to sort the order of responses, default: trust_score_desc

          page: page through results

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/exchanges/{id}/tickers",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "coin_ids": coin_ids,
                        "depth": depth,
                        "dex_pair_format": dex_pair_format,
                        "include_exchange_logo": include_exchange_logo,
                        "order": order,
                        "page": page,
                    },
                    ticker_get_params.TickerGetParams,
                ),
            ),
            cast_to=TickerGetResponse,
        )


class TickersResourceWithRawResponse:
    def __init__(self, tickers: TickersResource) -> None:
        self._tickers = tickers

        self.get = to_raw_response_wrapper(
            tickers.get,
        )


class AsyncTickersResourceWithRawResponse:
    def __init__(self, tickers: AsyncTickersResource) -> None:
        self._tickers = tickers

        self.get = async_to_raw_response_wrapper(
            tickers.get,
        )


class TickersResourceWithStreamingResponse:
    def __init__(self, tickers: TickersResource) -> None:
        self._tickers = tickers

        self.get = to_streamed_response_wrapper(
            tickers.get,
        )


class AsyncTickersResourceWithStreamingResponse:
    def __init__(self, tickers: AsyncTickersResource) -> None:
        self._tickers = tickers

        self.get = async_to_streamed_response_wrapper(
            tickers.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ....._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.onchain.networks.tokens import multi_get_addresses_params
from .....types.onchain.networks.tokens.multi_get_addresses_response import MultiGetAddressesResponse

__all__ = ["MultiResource", "AsyncMultiResource"]


class MultiResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MultiResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return MultiResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MultiResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return MultiResourceWithStreamingResponse(self)

    def get_addresses(
        self,
        addresses: str,
        *,
        network: str,
        include: Literal["top_pools"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MultiGetAddressesResponse:
        """
        This endpoint allows you to **query multiple tokens data based on the provided
        token contract addresses on a network**

        Args:
          include: attributes to include

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not network:
            raise ValueError(f"Expected a non-empty value for `network` but received {network!r}")
        if not addresses:
            raise ValueError(f"Expected a non-empty value for `addresses` but received {addresses!r}")
        return self._get(
            f"/onchain/networks/{network}/tokens/multi/{addresses}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"include": include}, multi_get_addresses_params.MultiGetAddressesParams),
            ),
            cast_to=MultiGetAddressesResponse,
        )


class AsyncMultiResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMultiResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMultiResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMultiResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncMultiResourceWithStreamingResponse(self)

    async def get_addresses(
        self,
        addresses: str,
        *,
        network: str,
        include: Literal["top_pools"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MultiGetAddressesResponse:
        """
        This endpoint allows you to **query multiple tokens data based on the provided
        token contract addresses on a network**

        Args:
          include: attributes to include

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not network:
            raise ValueError(f"Expected a non-empty value for `network` but received {network!r}")
        if not addresses:
            raise ValueError(f"Expected a non-empty value for `addresses` but received {addresses!r}")
        return await self._get(
            f"/onchain/networks/{network}/tokens/multi/{addresses}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"include": include}, multi_get_addresses_params.MultiGetAddressesParams
                ),
            ),
            cast_to=MultiGetAddressesResponse,
        )


class MultiResourceWithRawResponse:
    def __init__(self, multi: MultiResource) -> None:
        self._multi = multi

        self.get_addresses = to_raw_response_wrapper(
            multi.get_addresses,
        )


class AsyncMultiResourceWithRawResponse:
    def __init__(self, multi: AsyncMultiResource) -> None:
        self._multi = multi

        self.get_addresses = async_to_raw_response_wrapper(
            multi.get_addresses,
        )


class MultiResourceWithStreamingResponse:
    def __init__(self, multi: MultiResource) -> None:
        self._multi = multi

        self.get_addresses = to_streamed_response_wrapper(
            multi.get_addresses,
        )


class AsyncMultiResourceWithStreamingResponse:
    def __init__(self, multi: AsyncMultiResource) -> None:
        self._multi = multi

        self.get_addresses = async_to_streamed_response_wrapper(
            multi.get_addresses,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .list import (
    ListResource,
    AsyncListResource,
    ListResourceWithRawResponse,
    AsyncListResourceWithRawResponse,
    ListResourceWithStreamingResponse,
    AsyncListResourceWithStreamingResponse,
)
from .ohlc import (
    OhlcResource,
    AsyncOhlcResource,
    OhlcResourceWithRawResponse,
    AsyncOhlcResourceWithRawResponse,
    OhlcResourceWithStreamingResponse,
    AsyncOhlcResourceWithStreamingResponse,
)
from .coins import (
    CoinsResource,
    AsyncCoinsResource,
    CoinsResourceWithRawResponse,
    AsyncCoinsResourceWithRawResponse,
    CoinsResourceWithStreamingResponse,
    AsyncCoinsResourceWithStreamingResponse,
)
from .history import (
    HistoryResource,
    AsyncHistoryResource,
    HistoryResourceWithRawResponse,
    AsyncHistoryResourceWithRawResponse,
    HistoryResourceWithStreamingResponse,
    AsyncHistoryResourceWithStreamingResponse,
)
from .markets import (
    MarketsResource,
    AsyncMarketsResource,
    MarketsResourceWithRawResponse,
    AsyncMarketsResourceWithRawResponse,
    MarketsResourceWithStreamingResponse,
    AsyncMarketsResourceWithStreamingResponse,
)
from .tickers import (
    TickersResource,
    AsyncTickersResource,
    TickersResourceWithRawResponse,
    AsyncTickersResourceWithRawResponse,
    TickersResourceWithStreamingResponse,
    AsyncTickersResourceWithStreamingResponse,
)
from .contract import (
    ContractResource,
    AsyncContractResource,
    ContractResourceWithRawResponse,
    AsyncContractResourceWithRawResponse,
    ContractResourceWithStreamingResponse,
    AsyncContractResourceWithStreamingResponse,
)
from .categories import (
    CategoriesResource,
    AsyncCategoriesResource,
    CategoriesResourceWithRawResponse,
    AsyncCategoriesResourceWithRawResponse,
    CategoriesResourceWithStreamingResponse,
    AsyncCategoriesResourceWithStreamingResponse,
)
from .market_chart import (
    MarketChartResource,
    AsyncMarketChartResource,
    MarketChartResourceWithRawResponse,
    AsyncMarketChartResourceWithRawResponse,
    MarketChartResourceWithStreamingResponse,
    AsyncMarketChartResourceWithStreamingResponse,
)
from .top_gainers_losers import (
    TopGainersLosersResource,
    AsyncTopGainersLosersResource,
    TopGainersLosersResourceWithRawResponse,
    AsyncTopGainersLosersResourceWithRawResponse,
    TopGainersLosersResourceWithStreamingResponse,
    AsyncTopGainersLosersResourceWithStreamingResponse,
)
from .total_supply_chart import (
    TotalSupplyChartResource,
    AsyncTotalSupplyChartResource,
    TotalSupplyChartResourceWithRawResponse,
    AsyncTotalSupplyChartResourceWithRawResponse,
    TotalSupplyChartResourceWithStreamingResponse,
    AsyncTotalSupplyChartResourceWithStreamingResponse,
)
from .circulating_supply_chart import (
    CirculatingSupplyChartResource,
    AsyncCirculatingSupplyChartResource,
    CirculatingSupplyChartResourceWithRawResponse,
    AsyncCirculatingSupplyChartResourceWithRawResponse,
    CirculatingSupplyChartResourceWithStreamingResponse,
    AsyncCirculatingSupplyChartResourceWithStreamingResponse,
)

__all__ = [
    "CategoriesResource",
    "AsyncCategoriesResource",
    "CategoriesResourceWithRawResponse",
    "AsyncCategoriesResourceWithRawResponse",
    "CategoriesResourceWithStreamingResponse",
    "AsyncCategoriesResourceWithStreamingResponse",
    "ListResource",
    "AsyncListResource",
    "ListResourceWithRawResponse",
    "AsyncListResourceWithRawResponse",
    "ListResourceWithStreamingResponse",
    "AsyncListResourceWithStreamingResponse",
    "MarketsResource",
    "AsyncMarketsResource",
    "MarketsResourceWithRawResponse",
    "AsyncMarketsResourceWithRawResponse",
    "MarketsResourceWithStreamingResponse",
    "AsyncMarketsResourceWithStreamingResponse",
    "TopGainersLosersResource",
    "AsyncTopGainersLosersResource",
    "TopGainersLosersResourceWithRawResponse",
    "AsyncTopGainersLosersResourceWithRawResponse",
    "TopGainersLosersResourceWithStreamingResponse",
    "AsyncTopGainersLosersResourceWithStreamingResponse",
    "CirculatingSupplyChartResource",
    "AsyncCirculatingSupplyChartResource",
    "CirculatingSupplyChartResourceWithRawResponse",
    "AsyncCirculatingSupplyChartResourceWithRawResponse",
    "CirculatingSupplyChartResourceWithStreamingResponse",
    "AsyncCirculatingSupplyChartResourceWithStreamingResponse",
    "ContractResource",
    "AsyncContractResource",
    "ContractResourceWithRawResponse",
    "AsyncContractResourceWithRawResponse",
    "ContractResourceWithStreamingResponse",
    "AsyncContractResourceWithStreamingResponse",
    "HistoryResource",
    "AsyncHistoryResource",
    "HistoryResourceWithRawResponse",
    "AsyncHistoryResourceWithRawResponse",
    "HistoryResourceWithStreamingResponse",
    "AsyncHistoryResourceWithStreamingResponse",
    "MarketChartResource",
    "AsyncMarketChartResource",
    "MarketChartResourceWithRawResponse",
    "AsyncMarketChartResourceWithRawResponse",
    "MarketChartResourceWithStreamingResponse",
    "AsyncMarketChartResourceWithStreamingResponse",
    "OhlcResource",
    "AsyncOhlcResource",
    "OhlcResourceWithRawResponse",
    "AsyncOhlcResourceWithRawResponse",
    "OhlcResourceWithStreamingResponse",
    "AsyncOhlcResourceWithStreamingResponse",
    "TickersResource",
    "AsyncTickersResource",
    "TickersResourceWithRawResponse",
    "AsyncTickersResourceWithRawResponse",
    "TickersResourceWithStreamingResponse",
    "AsyncTickersResourceWithStreamingResponse",
    "TotalSupplyChartResource",
    "AsyncTotalSupplyChartResource",
    "TotalSupplyChartResourceWithRawResponse",
    "AsyncTotalSupplyChartResourceWithRawResponse",
    "TotalSupplyChartResourceWithStreamingResponse",
    "AsyncTotalSupplyChartResourceWithStreamingResponse",
    "CoinsResource",
    "AsyncCoinsResource",
    "CoinsResourceWithRawResponse",
    "AsyncCoinsResourceWithRawResponse",
    "CoinsResourceWithStreamingResponse",
    "AsyncCoinsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .list_get_params import ListGetParams as ListGetParams
from .ohlc_get_params import OhlcGetParams as OhlcGetParams
from .list_get_response import ListGetResponse as ListGetResponse
from .market_get_params import MarketGetParams as MarketGetParams
from .ohlc_get_response import OhlcGetResponse as OhlcGetResponse
from .ticker_get_params import TickerGetParams as TickerGetParams
from .history_get_params import HistoryGetParams as HistoryGetParams
from .category_get_params import CategoryGetParams as CategoryGetParams
from .market_get_response import MarketGetResponse as MarketGetResponse
from .ticker_get_response import TickerGetResponse as TickerGetResponse
from .history_get_response import HistoryGetResponse as HistoryGetResponse
from .category_get_response import CategoryGetResponse as CategoryGetResponse
from .contract_get_response import ContractGetResponse as ContractGetResponse
from .list_get_new_response import ListGetNewResponse as ListGetNewResponse
from .ohlc_get_range_params import OhlcGetRangeParams as OhlcGetRangeParams
from .market_chart_get_params import MarketChartGetParams as MarketChartGetParams
from .ohlc_get_range_response import OhlcGetRangeResponse as OhlcGetRangeResponse
from .market_chart_get_response import MarketChartGetResponse as MarketChartGetResponse
from .category_get_list_response import CategoryGetListResponse as CategoryGetListResponse
from .top_gainers_loser_get_params import TopGainersLoserGetParams as TopGainersLoserGetParams
from .market_chart_get_range_params import MarketChartGetRangeParams as MarketChartGetRangeParams
from .total_supply_chart_get_params import TotalSupplyChartGetParams as TotalSupplyChartGetParams
from .top_gainers_loser_get_response import TopGainersLoserGetResponse as TopGainersLoserGetResponse
from .market_chart_get_range_response import MarketChartGetRangeResponse as MarketChartGetRangeResponse
from .total_supply_chart_get_response import TotalSupplyChartGetResponse as TotalSupplyChartGetResponse
from .circulating_supply_chart_get_params import CirculatingSupplyChartGetParams as CirculatingSupplyChartGetParams
from .total_supply_chart_get_range_params import TotalSupplyChartGetRangeParams as TotalSupplyChartGetRangeParams
from .circulating_supply_chart_get_response import (
    CirculatingSupplyChartGetResponse as CirculatingSupplyChartGetResponse,
)
from .total_supply_chart_get_range_response import TotalSupplyChartGetRangeResponse as TotalSupplyChartGetRangeResponse
from .circulating_supply_chart_get_range_params import (
    CirculatingSupplyChartGetRangeParams as CirculatingSupplyChartGetRangeParams,
)
from .circulating_supply_chart_get_range_response import (
    CirculatingSupplyChartGetRangeResponse as CirculatingSupplyChartGetRangeResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .market_cap_chart_get_params import MarketCapChartGetParams as MarketCapChartGetParams
from .market_cap_chart_get_response import MarketCapChartGetResponse as MarketCapChartGetResponse
from .decentralized_finance_defi_get_response import (
    DecentralizedFinanceDefiGetResponse as DecentralizedFinanceDefiGetResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from .info_recently_updated import (
    InfoRecentlyUpdatedResource,
    AsyncInfoRecentlyUpdatedResource,
    InfoRecentlyUpdatedResourceWithRawResponse,
    AsyncInfoRecentlyUpdatedResourceWithRawResponse,
    InfoRecentlyUpdatedResourceWithStreamingResponse,
    AsyncInfoRecentlyUpdatedResourceWithStreamingResponse,
)

__all__ = ["TokensResource", "AsyncTokensResource"]


class TokensResource(SyncAPIResource):
    @cached_property
    def info_recently_updated(self) -> InfoRecentlyUpdatedResource:
        return InfoRecentlyUpdatedResource(self._client)

    @cached_property
    def with_raw_response(self) -> TokensResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return TokensResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TokensResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return TokensResourceWithStreamingResponse(self)


class AsyncTokensResource(AsyncAPIResource):
    @cached_property
    def info_recently_updated(self) -> AsyncInfoRecentlyUpdatedResource:
        return AsyncInfoRecentlyUpdatedResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTokensResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTokensResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTokensResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncTokensResourceWithStreamingResponse(self)


class TokensResourceWithRawResponse:
    def __init__(self, tokens: TokensResource) -> None:
        self._tokens = tokens

    @cached_property
    def info_recently_updated(self) -> InfoRecentlyUpdatedResourceWithRawResponse:
        return InfoRecentlyUpdatedResourceWithRawResponse(self._tokens.info_recently_updated)


class AsyncTokensResourceWithRawResponse:
    def __init__(self, tokens: AsyncTokensResource) -> None:
        self._tokens = tokens

    @cached_property
    def info_recently_updated(self) -> AsyncInfoRecentlyUpdatedResourceWithRawResponse:
        return AsyncInfoRecentlyUpdatedResourceWithRawResponse(self._tokens.info_recently_updated)


class TokensResourceWithStreamingResponse:
    def __init__(self, tokens: TokensResource) -> None:
        self._tokens = tokens

    @cached_property
    def info_recently_updated(self) -> InfoRecentlyUpdatedResourceWithStreamingResponse:
        return InfoRecentlyUpdatedResourceWithStreamingResponse(self._tokens.info_recently_updated)


class AsyncTokensResourceWithStreamingResponse:
    def __init__(self, tokens: AsyncTokensResource) -> None:
        self._tokens = tokens

    @cached_property
    def info_recently_updated(self) -> AsyncInfoRecentlyUpdatedResourceWithStreamingResponse:
        return AsyncInfoRecentlyUpdatedResourceWithStreamingResponse(self._tokens.info_recently_updated)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["ExchangeGetIDResponse", "Ticker"]


class Ticker(BaseModel):
    basis: Optional[float] = None
    """difference of derivative price and index price"""

    contract_type: Optional[str] = None
    """derivative contract type"""

    expired_at: Optional[str] = None

    funding_rate: Optional[float] = None
    """derivative funding rate"""

    index: Optional[float] = None
    """derivative underlying asset price"""

    index_id: Optional[str] = None
    """derivative underlying asset"""

    last_traded_at: Optional[float] = None
    """derivative last updated time"""

    market: Optional[str] = None
    """derivative market name"""

    open_interest: Optional[float] = None
    """derivative open interest"""

    price: Optional[str] = None
    """derivative ticker price"""

    price_percentage_change_24h: Optional[float] = None
    """derivative ticker price percentage change in 24 hours"""

    spread: Optional[float] = None
    """derivative bid ask spread"""

    symbol: Optional[str] = None
    """derivative ticker symbol"""

    volume_24h: Optional[float] = None
    """derivative volume in 24 hours"""


class ExchangeGetIDResponse(BaseModel):
    country: Optional[str] = None
    """derivatives exchange incorporated country"""

    description: Optional[str] = None
    """derivatives exchange description"""

    image: Optional[str] = None
    """derivatives exchange image url"""

    name: Optional[str] = None
    """derivatives exchange name"""

    number_of_futures_pairs: Optional[float] = None
    """number of futures pairs in the derivatives exchange"""

    number_of_perpetual_pairs: Optional[float] = None
    """number of perpetual pairs in the derivatives exchange"""

    open_interest_btc: Optional[float] = None
    """derivatives exchange open interest in BTC"""

    tickers: Optional[List[Ticker]] = None

    trade_volume_24h_btc: Optional[str] = None
    """derivatives exchange trade volume in BTC in 24 hours"""

    url: Optional[str] = None
    """derivatives exchange website url"""

    year_established: Optional[float] = None
    """derivatives exchange established year"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ....._models import BaseModel

__all__ = [
    "MultiGetAddressesResponse",
    "Data",
    "DataAttributes",
    "DataAttributesVolumeUsd",
    "DataRelationships",
    "DataRelationshipsTopPools",
    "DataRelationshipsTopPoolsData",
]


class DataAttributesVolumeUsd(BaseModel):
    h24: Optional[str] = None


class DataAttributes(BaseModel):
    address: Optional[str] = None

    coingecko_coin_id: Optional[str] = None

    decimals: Optional[int] = None

    fdv_usd: Optional[str] = None

    image_url: Optional[str] = None

    market_cap_usd: Optional[str] = None

    name: Optional[str] = None

    normalized_total_supply: Optional[str] = None

    price_usd: Optional[str] = None

    symbol: Optional[str] = None

    total_reserve_in_usd: Optional[str] = None

    total_supply: Optional[str] = None

    volume_usd: Optional[DataAttributesVolumeUsd] = None


class DataRelationshipsTopPoolsData(BaseModel):
    id: Optional[str] = None

    type: Optional[str] = None


class DataRelationshipsTopPools(BaseModel):
    data: Optional[List[DataRelationshipsTopPoolsData]] = None


class DataRelationships(BaseModel):
    top_pools: Optional[DataRelationshipsTopPools] = None


class Data(BaseModel):
    id: Optional[str] = None

    attributes: Optional[DataAttributes] = None

    relationships: Optional[DataRelationships] = None

    type: Optional[str] = None


class MultiGetAddressesResponse(BaseModel):
    data: Optional[List[Data]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["CategoryGetResponse", "Data", "DataAttributes", "DataAttributesVolumeChangePercentage"]


class DataAttributesVolumeChangePercentage(BaseModel):
    h1: Optional[str] = None

    h12: Optional[str] = None

    h24: Optional[str] = None

    h6: Optional[str] = None


class DataAttributes(BaseModel):
    description: Optional[str] = None

    fdv_usd: Optional[str] = None

    h24_tx_count: Optional[int] = None

    h24_volume_usd: Optional[str] = None

    name: Optional[str] = None

    reserve_in_usd: Optional[str] = None

    volume_change_percentage: Optional[DataAttributesVolumeChangePercentage] = None


class Data(BaseModel):
    id: Optional[str] = None

    attributes: Optional[DataAttributes] = None

    type: Optional[str] = None


class CategoryGetResponse(BaseModel):
    data: Optional[List[Data]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.companies.public_treasury_get_coin_id_response import PublicTreasuryGetCoinIDResponse

__all__ = ["PublicTreasuryResource", "AsyncPublicTreasuryResource"]


class PublicTreasuryResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PublicTreasuryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return PublicTreasuryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PublicTreasuryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return PublicTreasuryResourceWithStreamingResponse(self)

    def get_coin_id(
        self,
        coin_id: Literal["bitcoin", "ethereum"],
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PublicTreasuryGetCoinIDResponse:
        """
        This endpoint allows you **query public companies’ Bitcoin or Ethereum
        holdings**

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not coin_id:
            raise ValueError(f"Expected a non-empty value for `coin_id` but received {coin_id!r}")
        return self._get(
            f"/companies/public_treasury/{coin_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PublicTreasuryGetCoinIDResponse,
        )


class AsyncPublicTreasuryResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPublicTreasuryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPublicTreasuryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPublicTreasuryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncPublicTreasuryResourceWithStreamingResponse(self)

    async def get_coin_id(
        self,
        coin_id: Literal["bitcoin", "ethereum"],
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PublicTreasuryGetCoinIDResponse:
        """
        This endpoint allows you **query public companies’ Bitcoin or Ethereum
        holdings**

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not coin_id:
            raise ValueError(f"Expected a non-empty value for `coin_id` but received {coin_id!r}")
        return await self._get(
            f"/companies/public_treasury/{coin_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PublicTreasuryGetCoinIDResponse,
        )


class PublicTreasuryResourceWithRawResponse:
    def __init__(self, public_treasury: PublicTreasuryResource) -> None:
        self._public_treasury = public_treasury

        self.get_coin_id = to_raw_response_wrapper(
            public_treasury.get_coin_id,
        )


class AsyncPublicTreasuryResourceWithRawResponse:
    def __init__(self, public_treasury: AsyncPublicTreasuryResource) -> None:
        self._public_treasury = public_treasury

        self.get_coin_id = async_to_raw_response_wrapper(
            public_treasury.get_coin_id,
        )


class PublicTreasuryResourceWithStreamingResponse:
    def __init__(self, public_treasury: PublicTreasuryResource) -> None:
        self._public_treasury = public_treasury

        self.get_coin_id = to_streamed_response_wrapper(
            public_treasury.get_coin_id,
        )


class AsyncPublicTreasuryResourceWithStreamingResponse:
    def __init__(self, public_treasury: AsyncPublicTreasuryResource) -> None:
        self._public_treasury = public_treasury

        self.get_coin_id = async_to_streamed_response_wrapper(
            public_treasury.get_coin_id,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tokens import (
    TokensResource,
    AsyncTokensResource,
    TokensResourceWithRawResponse,
    AsyncTokensResourceWithRawResponse,
    TokensResourceWithStreamingResponse,
    AsyncTokensResourceWithStreamingResponse,
)
from .info_recently_updated import (
    InfoRecentlyUpdatedResource,
    AsyncInfoRecentlyUpdatedResource,
    InfoRecentlyUpdatedResourceWithRawResponse,
    AsyncInfoRecentlyUpdatedResourceWithRawResponse,
    InfoRecentlyUpdatedResourceWithStreamingResponse,
    AsyncInfoRecentlyUpdatedResourceWithStreamingResponse,
)

__all__ = [
    "InfoRecentlyUpdatedResource",
    "AsyncInfoRecentlyUpdatedResource",
    "InfoRecentlyUpdatedResourceWithRawResponse",
    "AsyncInfoRecentlyUpdatedResourceWithRawResponse",
    "InfoRecentlyUpdatedResourceWithStreamingResponse",
    "AsyncInfoRecentlyUpdatedResourceWithStreamingResponse",
    "TokensResource",
    "AsyncTokensResource",
    "TokensResourceWithRawResponse",
    "AsyncTokensResourceWithRawResponse",
    "TokensResourceWithStreamingResponse",
    "AsyncTokensResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from coingecko_sdk import Coingecko, AsyncCoingecko
from coingecko_sdk.types.nfts import TickerGetResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTickers:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_get(self, client: Coingecko) -> None:
        ticker = client.nfts.tickers.get(
            "pudgy-penguins",
        )
        assert_matches_type(TickerGetResponse, ticker, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_get(self, client: Coingecko) -> None:
        response = client.nfts.tickers.with_raw_response.get(
            "pudgy-penguins",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ticker = response.parse()
        assert_matches_type(TickerGetResponse, ticker, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_get(self, client: Coingecko) -> None:
        with client.nfts.tickers.with_streaming_response.get(
            "pudgy-penguins",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ticker = response.parse()
            assert_matches_type(TickerGetResponse, ticker, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_get(self, client: Coingecko) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.nfts.tickers.with_raw_response.get(
                "",
            )


class TestAsyncTickers:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_get(self, async_client: AsyncCoingecko) -> None:
        ticker = await async_client.nfts.tickers.get(
            "pudgy-penguins",
        )
        assert_matches_type(TickerGetResponse, ticker, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_get(self, async_client: AsyncCoingecko) -> None:
        response = await async_client.nfts.tickers.with_raw_response.get(
            "pudgy-penguins",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ticker = await response.parse()
        assert_matches_type(TickerGetResponse, ticker, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_get(self, async_client: AsyncCoingecko) -> None:
        async with async_client.nfts.tickers.with_streaming_response.get(
            "pudgy-penguins",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ticker = await response.parse()
            assert_matches_type(TickerGetResponse, ticker, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_get(self, async_client: AsyncCoingecko) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.nfts.tickers.with_raw_response.get(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime

from .._models import BaseModel

__all__ = [
    "CoinGetIDResponse",
    "CommunityData",
    "DetailPlatforms",
    "DeveloperData",
    "DeveloperDataCodeAdditionsDeletions4Weeks",
    "Image",
    "Links",
    "LinksReposURL",
    "MarketData",
    "MarketDataAth",
    "MarketDataAthChangePercentage",
    "MarketDataAthDate",
    "MarketDataAtl",
    "MarketDataAtlChangePercentage",
    "MarketDataAtlDate",
    "MarketDataCurrentPrice",
    "MarketDataFullyDilutedValuation",
    "MarketDataHigh24h",
    "MarketDataLow24h",
    "MarketDataMarketCap",
    "MarketDataMarketCapChange24hInCurrency",
    "MarketDataMarketCapChangePercentage24hInCurrency",
    "MarketDataPriceChangePercentage14dInCurrency",
    "MarketDataPriceChangePercentage1hInCurrency",
    "MarketDataPriceChangePercentage1yInCurrency",
    "MarketDataPriceChangePercentage200dInCurrency",
    "MarketDataPriceChangePercentage24hInCurrency",
    "MarketDataPriceChangePercentage30dInCurrency",
    "MarketDataPriceChangePercentage60dInCurrency",
    "MarketDataPriceChangePercentage7dInCurrency",
    "MarketDataTotalVolume",
    "Ticker",
    "TickerConvertedLast",
    "TickerConvertedVolume",
    "TickerMarket",
]


class CommunityData(BaseModel):
    facebook_likes: Optional[float] = None
    """coin facebook likes"""

    reddit_accounts_active_48h: Optional[float] = None
    """coin reddit active accounts in 48 hours"""

    reddit_average_comments_48h: Optional[float] = None
    """coin reddit average comments in 48 hours"""

    reddit_average_posts_48h: Optional[float] = None
    """coin reddit average posts in 48 hours"""

    reddit_subscribers: Optional[float] = None
    """coin reddit subscribers"""

    telegram_channel_user_count: Optional[float] = None
    """coin telegram channel user count"""


class DetailPlatforms(BaseModel):
    contract_address: Optional[str] = None
    """contract address on the platform"""

    decimal_place: Optional[float] = None
    """decimal places for the token"""


class DeveloperDataCodeAdditionsDeletions4Weeks(BaseModel):
    additions: Optional[float] = None

    deletions: Optional[float] = None


class DeveloperData(BaseModel):
    closed_issues: Optional[float] = None
    """coin repository closed issues"""

    code_additions_deletions_4_weeks: Optional[DeveloperDataCodeAdditionsDeletions4Weeks] = None
    """coin code additions and deletions in 4 weeks"""

    commit_count_4_weeks: Optional[float] = None
    """coin repository commit count in 4 weeks"""

    forks: Optional[float] = None
    """coin repository forks"""

    last_4_weeks_commit_activity_series: Optional[List[float]] = None
    """coin repository last 4 weeks commit activity series"""

    pull_request_contributors: Optional[float] = None
    """coin repository pull request contributors"""

    pull_requests_merged: Optional[float] = None
    """coin repository pull requests merged"""

    stars: Optional[float] = None
    """coin repository stars"""

    subscribers: Optional[float] = None
    """coin repository subscribers"""

    total_issues: Optional[float] = None
    """coin repository total issues"""


class Image(BaseModel):
    large: Optional[str] = None

    small: Optional[str] = None

    thumb: Optional[str] = None


class LinksReposURL(BaseModel):
    bitbucket: Optional[List[str]] = None
    """coin bitbucket repository url"""

    github: Optional[List[str]] = None
    """coin github repository url"""


class Links(BaseModel):
    announcement_url: Optional[List[str]] = None
    """coin announcement url"""

    bitcointalk_thread_identifier: Optional[str] = None
    """coin bitcointalk thread identifier"""

    blockchain_site: Optional[List[str]] = None
    """coin block explorer url"""

    chat_url: Optional[List[str]] = None
    """coin chat url"""

    facebook_username: Optional[str] = None
    """coin facebook username"""

    homepage: Optional[List[str]] = None
    """coin website url"""

    official_forum_url: Optional[List[str]] = None
    """coin official forum url"""

    repos_url: Optional[LinksReposURL] = None
    """coin repository url"""

    snapshot_url: Optional[str] = None
    """coin snapshot url"""

    subreddit_url: Optional[str] = None
    """coin subreddit url"""

    telegram_channel_identifier: Optional[str] = None
    """coin telegram channel identifier"""

    twitter_screen_name: Optional[str] = None
    """coin twitter handle"""

    whitepaper: Optional[List[str]] = None
    """coin whitepaper url"""


class MarketDataAth(BaseModel):
    btc: Optional[float] = None

    eur: Optional[float] = None

    usd: Optional[float] = None


class MarketDataAthChangePercentage(BaseModel):
    btc: Optional[float] = None

    eur: Optional[float] = None

    usd: Optional[float] = None


class MarketDataAthDate(BaseModel):
    btc: Optional[str] = None

    eur: Optional[str] = None

    usd: Optional[str] = None


class MarketDataAtl(BaseModel):
    btc: Optional[float] = None

    eur: Optional[float] = None

    usd: Optional[float] = None


class MarketDataAtlChangePercentage(BaseModel):
    btc: Optional[float] = None

    eur: Optional[float] = None

    usd: Optional[float] = None


class MarketDataAtlDate(BaseModel):
    btc: Optional[str] = None

    eur: Optional[str] = None

    usd: Optional[str] = None


class MarketDataCurrentPrice(BaseModel):
    btc: Optional[float] = None

    eur: Optional[float] = None

    usd: Optional[float] = None


class MarketDataFullyDilutedValuation(BaseModel):
    btc: Optional[float] = None

    eur: Optional[float] = None

    usd: Optional[float] = None


class MarketDataHigh24h(BaseModel):
    btc: Optional[float] = None

    eur: Optional[float] = None

    usd: Optional[float] = None


class MarketDataLow24h(BaseModel):
    btc: Optional[float] = None

    eur: Optional[float] = None

    usd: Optional[float] = None


class MarketDataMarketCap(BaseModel):
    btc: Optional[float] = None

    eur: Optional[float] = None

    usd: Optional[float] = None


class MarketDataMarketCapChange24hInCurrency(BaseModel):
    btc: Optional[float] = None

    eur: Optional[float] = None

    usd: Optional[float] = None


class MarketDataMarketCapChangePercentage24hInCurrency(BaseModel):
    btc: Optional[float] = None

    eur: Optional[float] = None

    usd: Optional[float] = None


class MarketDataPriceChangePercentage14dInCurrency(BaseModel):
    btc: Optional[float] = None

    eur: Optional[float] = None

    usd: Optional[float] = None


class MarketDataPriceChangePercentage1hInCurrency(BaseModel):
    btc: Optional[float] = None

    eur: Optional[float] = None

    usd: Optional[float] = None


class MarketDataPriceChangePercentage1yInCurrency(BaseModel):
    btc: Optional[float] = None

    eur: Optional[float] = None

    usd: Optional[float] = None


class MarketDataPriceChangePercentage200dInCurrency(BaseModel):
    btc: Optional[float] = None

    eur: Optional[float] = None

    usd: Optional[float] = None


class MarketDataPriceChangePercentage24hInCurrency(BaseModel):
    btc: Optional[float] = None

    eur: Optional[float] = None

    usd: Optional[float] = None


class MarketDataPriceChangePercentage30dInCurrency(BaseModel):
    btc: Optional[float] = None

    eur: Optional[float] = None

    usd: Optional[float] = None


class MarketDataPriceChangePercentage60dInCurrency(BaseModel):
    btc: Optional[float] = None

    eur: Optional[float] = None

    usd: Optional[float] = None


class MarketDataPriceChangePercentage7dInCurrency(BaseModel):
    btc: Optional[float] = None

    eur: Optional[float] = None

    usd: Optional[float] = None


class MarketDataTotalVolume(BaseModel):
    btc: Optional[float] = None

    eur: Optional[float] = None

    usd: Optional[float] = None


class MarketData(BaseModel):
    ath: Optional[MarketDataAth] = None
    """coin all time high (ATH) in currency"""

    ath_change_percentage: Optional[MarketDataAthChangePercentage] = None
    """coin all time high (ATH) change in percentage"""

    ath_date: Optional[MarketDataAthDate] = None
    """coin all time high (ATH) date"""

    atl: Optional[MarketDataAtl] = None
    """coin all time low (atl) in currency"""

    atl_change_percentage: Optional[MarketDataAtlChangePercentage] = None
    """coin all time low (atl) change in percentage"""

    atl_date: Optional[MarketDataAtlDate] = None
    """coin all time low (atl) date"""

    circulating_supply: Optional[float] = None
    """coin circulating supply"""

    current_price: Optional[MarketDataCurrentPrice] = None
    """coin current price in currency"""

    fdv_to_tvl_ratio: Optional[float] = None
    """fully diluted valuation to total value locked ratio"""

    fully_diluted_valuation: Optional[MarketDataFullyDilutedValuation] = None
    """coin fully diluted valuation (fdv) in currency"""

    high_24h: Optional[MarketDataHigh24h] = None
    """coin 24hr price high in currency"""

    last_updated: Optional[datetime] = None
    """coin market data last updated timestamp"""

    low_24h: Optional[MarketDataLow24h] = None
    """coin 24hr price low in currency"""

    market_cap: Optional[MarketDataMarketCap] = None
    """coin market cap in currency"""

    market_cap_change_24h: Optional[float] = None
    """coin 24hr market cap change in currency"""

    market_cap_change_24h_in_currency: Optional[MarketDataMarketCapChange24hInCurrency] = None
    """coin 24hr market cap change in currency"""

    market_cap_change_percentage_24h: Optional[float] = None
    """coin 24hr market cap change in percentage"""

    market_cap_change_percentage_24h_in_currency: Optional[MarketDataMarketCapChangePercentage24hInCurrency] = None
    """coin 24hr market cap change in percentage"""

    market_cap_fdv_ratio: Optional[float] = None
    """market cap to fully diluted valuation ratio"""

    market_cap_rank: Optional[float] = None
    """coin rank by market cap"""

    max_supply: Optional[float] = None
    """coin max supply"""

    mcap_to_tvl_ratio: Optional[float] = None
    """market cap to total value locked ratio"""

    price_change_24h: Optional[float] = None
    """coin 24hr price change in currency"""

    price_change_percentage_14d: Optional[float] = None
    """coin 14d price change in percentage"""

    price_change_percentage_14d_in_currency: Optional[MarketDataPriceChangePercentage14dInCurrency] = None
    """coin 14d price change in currency"""

    price_change_percentage_1h_in_currency: Optional[MarketDataPriceChangePercentage1hInCurrency] = None
    """coin 1h price change in currency"""

    price_change_percentage_1y: Optional[float] = None
    """coin 1y price change in percentage"""

    price_change_percentage_1y_in_currency: Optional[MarketDataPriceChangePercentage1yInCurrency] = None
    """coin 1y price change in currency"""

    price_change_percentage_200d: Optional[float] = None
    """coin 200d price change in percentage"""

    price_change_percentage_200d_in_currency: Optional[MarketDataPriceChangePercentage200dInCurrency] = None
    """coin 200d price change in currency"""

    price_change_percentage_24h: Optional[float] = None
    """coin 24hr price change in percentage"""

    price_change_percentage_24h_in_currency: Optional[MarketDataPriceChangePercentage24hInCurrency] = None
    """coin 24hr price change in currency"""

    price_change_percentage_30d: Optional[float] = None
    """coin 30d price change in percentage"""

    price_change_percentage_30d_in_currency: Optional[MarketDataPriceChangePercentage30dInCurrency] = None
    """coin 30d price change in currency"""

    price_change_percentage_60d: Optional[float] = None
    """coin 60d price change in percentage"""

    price_change_percentage_60d_in_currency: Optional[MarketDataPriceChangePercentage60dInCurrency] = None
    """coin 60d price change in currency"""

    price_change_percentage_7d: Optional[float] = None
    """coin 7d price change in percentage"""

    price_change_percentage_7d_in_currency: Optional[MarketDataPriceChangePercentage7dInCurrency] = None
    """coin 7d price change in currency"""

    roi: Optional[float] = None
    """coin return on investment"""

    total_supply: Optional[float] = None
    """coin total supply"""

    total_value_locked: Optional[float] = None
    """total value locked"""

    total_volume: Optional[MarketDataTotalVolume] = None
    """coin total trading volume in currency"""


class TickerConvertedLast(BaseModel):
    btc: Optional[float] = None

    eth: Optional[float] = None

    usd: Optional[float] = None


class TickerConvertedVolume(BaseModel):
    btc: Optional[float] = None

    eth: Optional[float] = None

    usd: Optional[float] = None


class TickerMarket(BaseModel):
    has_trading_incentive: Optional[bool] = None
    """coin ticker exchange trading incentive"""

    identifier: Optional[str] = None
    """coin ticker exchange identifier"""

    name: Optional[str] = None
    """coin ticker exchange name"""


class Ticker(BaseModel):
    base: Optional[str] = None
    """coin ticker base currency"""

    bid_ask_spread_percentage: Optional[float] = None
    """coin ticker bid ask spread percentage"""

    coin_id: Optional[str] = None
    """coin ticker base currency coin ID"""

    converted_last: Optional[TickerConvertedLast] = None
    """coin ticker converted last price"""

    converted_volume: Optional[TickerConvertedVolume] = None
    """coin ticker converted volume"""

    is_anomaly: Optional[bool] = None
    """coin ticker anomaly"""

    is_stale: Optional[bool] = None
    """coin ticker stale"""

    last: Optional[float] = None
    """coin ticker last price"""

    last_fetch_at: Optional[datetime] = None
    """coin ticker last fetch timestamp"""

    last_traded_at: Optional[datetime] = None
    """coin ticker last traded timestamp"""

    market: Optional[TickerMarket] = None
    """coin ticker exchange"""

    target: Optional[str] = None
    """coin ticker target currency"""

    target_coin_id: Optional[str] = None
    """coin ticker target currency coin ID"""

    timestamp: Optional[datetime] = None
    """coin ticker timestamp"""

    token_info_url: Optional[str] = None
    """coin ticker token info url"""

    trade_url: Optional[str] = None
    """coin ticker trade url"""

    trust_score: Optional[str] = None
    """coin ticker trust score"""

    volume: Optional[float] = None
    """coin ticker volume"""


class CoinGetIDResponse(BaseModel):
    id: Optional[str] = None
    """coin ID"""

    additional_notices: Optional[List[str]] = None
    """additional notices"""

    asset_platform_id: Optional[str] = None
    """coin asset platform ID"""

    block_time_in_minutes: Optional[float] = None
    """blockchain block time in minutes"""

    categories: Optional[List[str]] = None
    """coin categories"""

    community_data: Optional[CommunityData] = None
    """coin community data"""

    country_origin: Optional[str] = None
    """coin country of origin"""

    description: Optional[Dict[str, str]] = None
    """coin description"""

    detail_platforms: Optional[Dict[str, DetailPlatforms]] = None
    """detailed coin asset platform and contract address"""

    developer_data: Optional[DeveloperData] = None
    """coin developer data"""

    genesis_date: Optional[datetime] = None
    """coin genesis date"""

    hashing_algorithm: Optional[str] = None
    """blockchain hashing algorithm"""

    image: Optional[Image] = None
    """coin image url"""

    last_updated: Optional[datetime] = None
    """coin last updated timestamp"""

    links: Optional[Links] = None
    """links"""

    localization: Optional[Dict[str, str]] = None
    """coin name localization"""

    market_cap_rank: Optional[float] = None
    """coin rank by market cap"""

    market_data: Optional[MarketData] = None
    """coin market data"""

    name: Optional[str] = None
    """coin name"""

    platforms: Optional[Dict[str, str]] = None
    """coin asset platform and contract address"""

    preview_listing: Optional[bool] = None
    """preview listing coin"""

    public_notice: Optional[str] = None
    """public notice"""

    sentiment_votes_down_percentage: Optional[float] = None
    """coin sentiment votes down percentage"""

    sentiment_votes_up_percentage: Optional[float] = None
    """coin sentiment votes up percentage"""

    status_updates: Optional[List[str]] = None
    """coin status updates"""

    symbol: Optional[str] = None
    """coin symbol"""

    tickers: Optional[List[Ticker]] = None
    """coin tickers"""

    web_slug: Optional[str] = None
    """coin web slug"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .search import (
    SearchResource,
    AsyncSearchResource,
    SearchResourceWithRawResponse,
    AsyncSearchResourceWithRawResponse,
    SearchResourceWithStreamingResponse,
    AsyncSearchResourceWithStreamingResponse,
)
from .trending import (
    TrendingResource,
    AsyncTrendingResource,
    TrendingResourceWithRawResponse,
    AsyncTrendingResourceWithRawResponse,
    TrendingResourceWithStreamingResponse,
    AsyncTrendingResourceWithStreamingResponse,
)

__all__ = [
    "TrendingResource",
    "AsyncTrendingResource",
    "TrendingResourceWithRawResponse",
    "AsyncTrendingResourceWithRawResponse",
    "TrendingResourceWithStreamingResponse",
    "AsyncTrendingResourceWithStreamingResponse",
    "SearchResource",
    "AsyncSearchResource",
    "SearchResourceWithRawResponse",
    "AsyncSearchResourceWithRawResponse",
    "SearchResourceWithStreamingResponse",
    "AsyncSearchResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["MarketCapChartGetParams"]


class MarketCapChartGetParams(TypedDict, total=False):
    days: Required[Literal["1", "7", "14", "30", "90", "180", "365", "max"]]
    """data up to number of days ago Valid values: any integer"""

    vs_currency: str
    """
    target currency of market cap, default: usd \\**refers to
    [`/simple/supported_vs_currencies`](/reference/simple-supported-currencies)
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import asset_platform_get_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.asset_platform_get_response import AssetPlatformGetResponse

__all__ = ["AssetPlatformsResource", "AsyncAssetPlatformsResource"]


class AssetPlatformsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AssetPlatformsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AssetPlatformsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AssetPlatformsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AssetPlatformsResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        filter: Literal["nft"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AssetPlatformGetResponse:
        """
        This endpoint allows you to **query all the asset platforms on CoinGecko**

        Args:
          filter: apply relevant filters to results

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/asset_platforms",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"filter": filter}, asset_platform_get_params.AssetPlatformGetParams),
            ),
            cast_to=AssetPlatformGetResponse,
        )


class AsyncAssetPlatformsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAssetPlatformsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAssetPlatformsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAssetPlatformsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncAssetPlatformsResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        filter: Literal["nft"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AssetPlatformGetResponse:
        """
        This endpoint allows you to **query all the asset platforms on CoinGecko**

        Args:
          filter: apply relevant filters to results

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/asset_platforms",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"filter": filter}, asset_platform_get_params.AssetPlatformGetParams),
            ),
            cast_to=AssetPlatformGetResponse,
        )


class AssetPlatformsResourceWithRawResponse:
    def __init__(self, asset_platforms: AssetPlatformsResource) -> None:
        self._asset_platforms = asset_platforms

        self.get = to_raw_response_wrapper(
            asset_platforms.get,
        )


class AsyncAssetPlatformsResourceWithRawResponse:
    def __init__(self, asset_platforms: AsyncAssetPlatformsResource) -> None:
        self._asset_platforms = asset_platforms

        self.get = async_to_raw_response_wrapper(
            asset_platforms.get,
        )


class AssetPlatformsResourceWithStreamingResponse:
    def __init__(self, asset_platforms: AssetPlatformsResource) -> None:
        self._asset_platforms = asset_platforms

        self.get = to_streamed_response_wrapper(
            asset_platforms.get,
        )


class AsyncAssetPlatformsResourceWithStreamingResponse:
    def __init__(self, asset_platforms: AsyncAssetPlatformsResource) -> None:
        self._asset_platforms = asset_platforms

        self.get = async_to_streamed_response_wrapper(
            asset_platforms.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["CategoryGetListResponse"]


class CategoryGetListResponse(BaseModel):
    category_id: Optional[str] = None
    """category ID"""

    name: Optional[str] = None
    """category name"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import TypeAlias

from ..._models import BaseModel

__all__ = ["TopGainersLoserGetResponse", "TopGainersLoserGetResponseItem"]


class TopGainersLoserGetResponseItem(BaseModel):
    id: Optional[str] = None
    """coin ID"""

    image: Optional[str] = None
    """coin image url"""

    market_cap_rank: Optional[float] = None
    """coin rank by market cap"""

    name: Optional[str] = None
    """coin name"""

    symbol: Optional[str] = None
    """coin symbol"""

    usd: Optional[float] = None
    """coin price in USD"""

    usd_1y_change: Optional[float] = None
    """coin 1 year change in USD"""

    usd_24h_vol: Optional[float] = None
    """coin 24hr volume in USD"""


TopGainersLoserGetResponse: TypeAlias = List[TopGainersLoserGetResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["DerivativeGetResponse"]


class DerivativeGetResponse(BaseModel):
    basis: Optional[float] = None
    """difference of derivative price and index price"""

    contract_type: Optional[str] = None
    """derivative contract type"""

    expired_at: Optional[str] = None

    funding_rate: Optional[float] = None
    """derivative funding rate"""

    index: Optional[float] = None
    """derivative underlying asset price"""

    index_id: Optional[str] = None
    """derivative underlying asset"""

    last_traded_at: Optional[float] = None
    """derivative last updated time"""

    market: Optional[str] = None
    """derivative market name"""

    open_interest: Optional[float] = None
    """derivative open interest"""

    price: Optional[str] = None
    """derivative ticker price"""

    price_percentage_change_24h: Optional[float] = None
    """derivative ticker price percentage change in 24 hours"""

    spread: Optional[float] = None
    """derivative bid ask spread"""

    symbol: Optional[str] = None
    """derivative ticker symbol"""

    volume_24h: Optional[float] = None
    """derivative volume in 24 hours"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .market_chart_get_params import MarketChartGetParams as MarketChartGetParams
from .market_chart_get_response import MarketChartGetResponse as MarketChartGetResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from .._models import BaseModel

__all__ = ["ExchangeRateGetResponse", "Rates"]


class Rates(BaseModel):
    name: Optional[str] = None
    """name of the currency"""

    type: Optional[str] = None
    """type of the currency"""

    unit: Optional[str] = None
    """unit of the currency"""

    value: Optional[float] = None
    """value of the currency"""


class ExchangeRateGetResponse(BaseModel):
    rates: Optional[Dict[str, Rates]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from ....._models import BaseModel

__all__ = [
    "InfoGetResponse",
    "Data",
    "DataAttributes",
    "DataAttributesGtScoreDetails",
    "DataAttributesHolders",
    "DataAttributesHoldersDistributionPercentage",
    "DataAttributesImage",
]


class DataAttributesGtScoreDetails(BaseModel):
    creation: Optional[float] = None

    holders: Optional[float] = None

    info: Optional[float] = None

    pool: Optional[float] = None

    transaction: Optional[float] = None


class DataAttributesHoldersDistributionPercentage(BaseModel):
    dist_11_30: Optional[float] = FieldInfo(alias="11_30", default=None)

    dist_31_50: Optional[float] = FieldInfo(alias="31_50", default=None)

    rest: Optional[float] = None

    top_10: Optional[float] = None


class DataAttributesHolders(BaseModel):
    count: Optional[int] = None

    distribution_percentage: Optional[DataAttributesHoldersDistributionPercentage] = None

    last_updated: Optional[str] = None


class DataAttributesImage(BaseModel):
    large: Optional[str] = None

    small: Optional[str] = None

    thumb: Optional[str] = None


class DataAttributes(BaseModel):
    address: Optional[str] = None

    categories: Optional[List[str]] = None

    coingecko_coin_id: Optional[str] = None

    description: Optional[str] = None

    discord_url: Optional[str] = None

    freeze_authority: Optional[str] = None

    gt_categories_id: Optional[List[str]] = None

    gt_score: Optional[float] = None

    gt_score_details: Optional[DataAttributesGtScoreDetails] = None

    holders: Optional[DataAttributesHolders] = None

    image: Optional[DataAttributesImage] = None

    image_url: Optional[str] = None

    mint_authority: Optional[str] = None

    name: Optional[str] = None

    symbol: Optional[str] = None

    telegram_handle: Optional[str] = None

    twitter_handle: Optional[str] = None

    websites: Optional[List[str]] = None


class Data(BaseModel):
    id: Optional[str] = None

    attributes: Optional[DataAttributes] = None

    type: Optional[str] = None


class InfoGetResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.coins import market_get_params
from ..._base_client import make_request_options
from ...types.coins.market_get_response import MarketGetResponse

__all__ = ["MarketsResource", "AsyncMarketsResource"]


class MarketsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MarketsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return MarketsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MarketsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return MarketsResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        vs_currency: str,
        category: str | NotGiven = NOT_GIVEN,
        ids: str | NotGiven = NOT_GIVEN,
        include_tokens: Literal["top", "all"] | NotGiven = NOT_GIVEN,
        locale: Literal[
            "ar",
            "bg",
            "cs",
            "da",
            "de",
            "el",
            "en",
            "es",
            "fi",
            "fr",
            "he",
            "hi",
            "hr",
            "hu",
            "id",
            "it",
            "ja",
            "ko",
            "lt",
            "nl",
            "no",
            "pl",
            "pt",
            "ro",
            "ru",
            "sk",
            "sl",
            "sv",
            "th",
            "tr",
            "uk",
            "vi",
            "zh",
            "zh-tw",
        ]
        | NotGiven = NOT_GIVEN,
        names: str | NotGiven = NOT_GIVEN,
        order: Literal["market_cap_asc", "market_cap_desc", "volume_asc", "volume_desc", "id_asc", "id_desc"]
        | NotGiven = NOT_GIVEN,
        page: float | NotGiven = NOT_GIVEN,
        per_page: float | NotGiven = NOT_GIVEN,
        precision: Literal[
            "full",
            "0",
            "1",
            "2",
            "3",
            "4",
            "5",
            "6",
            "7",
            "8",
            "9",
            "10",
            "11",
            "12",
            "13",
            "14",
            "15",
            "16",
            "17",
            "18",
        ]
        | NotGiven = NOT_GIVEN,
        price_change_percentage: str | NotGiven = NOT_GIVEN,
        sparkline: bool | NotGiven = NOT_GIVEN,
        symbols: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MarketGetResponse:
        """
        This endpoint allows you to **query all the supported coins with price, market
        cap, volume and market related data**

        Args:
          vs_currency: target currency of coins and market data \\**refers to
              [`/simple/supported_vs_currencies`](/reference/simple-supported-currencies).

          category: filter based on coins' category \\**refers to
              [`/coins/categories/list`](/reference/coins-categories-list).

          ids: coins' IDs, comma-separated if querying more than 1 coin. \\**refers to
              [`/coins/list`](/reference/coins-list).

          include_tokens: for `symbols` lookups, specify `all` to include all matching tokens Default
              `top` returns top-ranked tokens (by market cap or volume)

          locale: language background, default: en

          names: coins' names, comma-separated if querying more than 1 coin.

          order: sort result by field, default: market_cap_desc

          page: page through results, default: 1

          per_page: total results per page, default: 100 Valid values: 1...250

          precision: decimal place for currency price value

          price_change_percentage: include price change percentage timeframe, comma-separated if query more than 1
              price change percentage timeframe Valid values: 1h, 24h, 7d, 14d, 30d, 200d, 1y

          sparkline: include sparkline 7 days data, default: false

          symbols: coins' symbols, comma-separated if querying more than 1 coin.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/coins/markets",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "vs_currency": vs_currency,
                        "category": category,
                        "ids": ids,
                        "include_tokens": include_tokens,
                        "locale": locale,
                        "names": names,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                        "precision": precision,
                        "price_change_percentage": price_change_percentage,
                        "sparkline": sparkline,
                        "symbols": symbols,
                    },
                    market_get_params.MarketGetParams,
                ),
            ),
            cast_to=MarketGetResponse,
        )


class AsyncMarketsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMarketsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMarketsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMarketsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncMarketsResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        vs_currency: str,
        category: str | NotGiven = NOT_GIVEN,
        ids: str | NotGiven = NOT_GIVEN,
        include_tokens: Literal["top", "all"] | NotGiven = NOT_GIVEN,
        locale: Literal[
            "ar",
            "bg",
            "cs",
            "da",
            "de",
            "el",
            "en",
            "es",
            "fi",
            "fr",
            "he",
            "hi",
            "hr",
            "hu",
            "id",
            "it",
            "ja",
            "ko",
            "lt",
            "nl",
            "no",
            "pl",
            "pt",
            "ro",
            "ru",
            "sk",
            "sl",
            "sv",
            "th",
            "tr",
            "uk",
            "vi",
            "zh",
            "zh-tw",
        ]
        | NotGiven = NOT_GIVEN,
        names: str | NotGiven = NOT_GIVEN,
        order: Literal["market_cap_asc", "market_cap_desc", "volume_asc", "volume_desc", "id_asc", "id_desc"]
        | NotGiven = NOT_GIVEN,
        page: float | NotGiven = NOT_GIVEN,
        per_page: float | NotGiven = NOT_GIVEN,
        precision: Literal[
            "full",
            "0",
            "1",
            "2",
            "3",
            "4",
            "5",
            "6",
            "7",
            "8",
            "9",
            "10",
            "11",
            "12",
            "13",
            "14",
            "15",
            "16",
            "17",
            "18",
        ]
        | NotGiven = NOT_GIVEN,
        price_change_percentage: str | NotGiven = NOT_GIVEN,
        sparkline: bool | NotGiven = NOT_GIVEN,
        symbols: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MarketGetResponse:
        """
        This endpoint allows you to **query all the supported coins with price, market
        cap, volume and market related data**

        Args:
          vs_currency: target currency of coins and market data \\**refers to
              [`/simple/supported_vs_currencies`](/reference/simple-supported-currencies).

          category: filter based on coins' category \\**refers to
              [`/coins/categories/list`](/reference/coins-categories-list).

          ids: coins' IDs, comma-separated if querying more than 1 coin. \\**refers to
              [`/coins/list`](/reference/coins-list).

          include_tokens: for `symbols` lookups, specify `all` to include all matching tokens Default
              `top` returns top-ranked tokens (by market cap or volume)

          locale: language background, default: en

          names: coins' names, comma-separated if querying more than 1 coin.

          order: sort result by field, default: market_cap_desc

          page: page through results, default: 1

          per_page: total results per page, default: 100 Valid values: 1...250

          precision: decimal place for currency price value

          price_change_percentage: include price change percentage timeframe, comma-separated if query more than 1
              price change percentage timeframe Valid values: 1h, 24h, 7d, 14d, 30d, 200d, 1y

          sparkline: include sparkline 7 days data, default: false

          symbols: coins' symbols, comma-separated if querying more than 1 coin.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/coins/markets",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "vs_currency": vs_currency,
                        "category": category,
                        "ids": ids,
                        "include_tokens": include_tokens,
                        "locale": locale,
                        "names": names,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                        "precision": precision,
                        "price_change_percentage": price_change_percentage,
                        "sparkline": sparkline,
                        "symbols": symbols,
                    },
                    market_get_params.MarketGetParams,
                ),
            ),
            cast_to=MarketGetResponse,
        )


class MarketsResourceWithRawResponse:
    def __init__(self, markets: MarketsResource) -> None:
        self._markets = markets

        self.get = to_raw_response_wrapper(
            markets.get,
        )


class AsyncMarketsResourceWithRawResponse:
    def __init__(self, markets: AsyncMarketsResource) -> None:
        self._markets = markets

        self.get = async_to_raw_response_wrapper(
            markets.get,
        )


class MarketsResourceWithStreamingResponse:
    def __init__(self, markets: MarketsResource) -> None:
        self._markets = markets

        self.get = to_streamed_response_wrapper(
            markets.get,
        )


class AsyncMarketsResourceWithStreamingResponse:
    def __init__(self, markets: AsyncMarketsResource) -> None:
        self._markets = markets

        self.get = async_to_streamed_response_wrapper(
            markets.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["TopHolderGetParams"]


class TopHolderGetParams(TypedDict, total=False):
    network: Required[str]

    holders: str
    """
    number of top token holders to return, you may use any integer or `max` Default
    value: 10
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["MarketChartGetResponse"]


class MarketChartGetResponse(BaseModel):
    market_caps: Optional[List[List[float]]] = None

    prices: Optional[List[List[float]]] = None

    total_volumes: Optional[List[List[float]]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from typing_extensions import TypeAlias

from ..._models import BaseModel

__all__ = ["ListGetResponse", "ListGetResponseItem"]


class ListGetResponseItem(BaseModel):
    id: Optional[str] = None
    """coin ID"""

    name: Optional[str] = None
    """coin name"""

    platforms: Optional[Dict[str, str]] = None
    """coin asset platform and contract address"""

    symbol: Optional[str] = None
    """coin symbol"""


ListGetResponse: TypeAlias = List[ListGetResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tickers import (
    TickersResource,
    AsyncTickersResource,
    TickersResourceWithRawResponse,
    AsyncTickersResourceWithRawResponse,
    TickersResourceWithStreamingResponse,
    AsyncTickersResourceWithStreamingResponse,
)
from .exchanges import (
    ExchangesResource,
    AsyncExchangesResource,
    ExchangesResourceWithRawResponse,
    AsyncExchangesResourceWithRawResponse,
    ExchangesResourceWithStreamingResponse,
    AsyncExchangesResourceWithStreamingResponse,
)
from .volume_chart import (
    VolumeChartResource,
    AsyncVolumeChartResource,
    VolumeChartResourceWithRawResponse,
    AsyncVolumeChartResourceWithRawResponse,
    VolumeChartResourceWithStreamingResponse,
    AsyncVolumeChartResourceWithStreamingResponse,
)

__all__ = [
    "TickersResource",
    "AsyncTickersResource",
    "TickersResourceWithRawResponse",
    "AsyncTickersResourceWithRawResponse",
    "TickersResourceWithStreamingResponse",
    "AsyncTickersResourceWithStreamingResponse",
    "VolumeChartResource",
    "AsyncVolumeChartResource",
    "VolumeChartResourceWithRawResponse",
    "AsyncVolumeChartResourceWithRawResponse",
    "VolumeChartResourceWithStreamingResponse",
    "AsyncVolumeChartResourceWithStreamingResponse",
    "ExchangesResource",
    "AsyncExchangesResource",
    "ExchangesResourceWithRawResponse",
    "AsyncExchangesResourceWithRawResponse",
    "ExchangesResourceWithStreamingResponse",
    "AsyncExchangesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["TrendingGetParams"]


class TrendingGetParams(TypedDict, total=False):
    show_max: str
    """
    show max number of results available for the given type Available values:
    `coins`, `nfts`, `categories` Example: `coins` or `coins,nfts,categories`
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ....._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.onchain.simple.networks import token_price_get_addresses_params
from .....types.onchain.simple.networks.token_price_get_addresses_response import TokenPriceGetAddressesResponse

__all__ = ["TokenPriceResource", "AsyncTokenPriceResource"]


class TokenPriceResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TokenPriceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return TokenPriceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TokenPriceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return TokenPriceResourceWithStreamingResponse(self)

    def get_addresses(
        self,
        addresses: str,
        *,
        network: str,
        include_24hr_price_change: bool | NotGiven = NOT_GIVEN,
        include_24hr_vol: bool | NotGiven = NOT_GIVEN,
        include_market_cap: bool | NotGiven = NOT_GIVEN,
        include_total_reserve_in_usd: bool | NotGiven = NOT_GIVEN,
        mcap_fdv_fallback: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TokenPriceGetAddressesResponse:
        """
        This endpoint allows you to **get token price based on the provided token
        contract address on a network**

        Args:
          include_24hr_price_change: include 24hr price change, default: false

          include_24hr_vol: include 24hr volume, default: false

          include_market_cap: include market capitalization, default: false

          include_total_reserve_in_usd: include total reserve in USD, default: false

          mcap_fdv_fallback: return FDV if market cap is not available, default: false

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not network:
            raise ValueError(f"Expected a non-empty value for `network` but received {network!r}")
        if not addresses:
            raise ValueError(f"Expected a non-empty value for `addresses` but received {addresses!r}")
        return self._get(
            f"/onchain/simple/networks/{network}/token_price/{addresses}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "include_24hr_price_change": include_24hr_price_change,
                        "include_24hr_vol": include_24hr_vol,
                        "include_market_cap": include_market_cap,
                        "include_total_reserve_in_usd": include_total_reserve_in_usd,
                        "mcap_fdv_fallback": mcap_fdv_fallback,
                    },
                    token_price_get_addresses_params.TokenPriceGetAddressesParams,
                ),
            ),
            cast_to=TokenPriceGetAddressesResponse,
        )


class AsyncTokenPriceResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTokenPriceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTokenPriceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTokenPriceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncTokenPriceResourceWithStreamingResponse(self)

    async def get_addresses(
        self,
        addresses: str,
        *,
        network: str,
        include_24hr_price_change: bool | NotGiven = NOT_GIVEN,
        include_24hr_vol: bool | NotGiven = NOT_GIVEN,
        include_market_cap: bool | NotGiven = NOT_GIVEN,
        include_total_reserve_in_usd: bool | NotGiven = NOT_GIVEN,
        mcap_fdv_fallback: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TokenPriceGetAddressesResponse:
        """
        This endpoint allows you to **get token price based on the provided token
        contract address on a network**

        Args:
          include_24hr_price_change: include 24hr price change, default: false

          include_24hr_vol: include 24hr volume, default: false

          include_market_cap: include market capitalization, default: false

          include_total_reserve_in_usd: include total reserve in USD, default: false

          mcap_fdv_fallback: return FDV if market cap is not available, default: false

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not network:
            raise ValueError(f"Expected a non-empty value for `network` but received {network!r}")
        if not addresses:
            raise ValueError(f"Expected a non-empty value for `addresses` but received {addresses!r}")
        return await self._get(
            f"/onchain/simple/networks/{network}/token_price/{addresses}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "include_24hr_price_change": include_24hr_price_change,
                        "include_24hr_vol": include_24hr_vol,
                        "include_market_cap": include_market_cap,
                        "include_total_reserve_in_usd": include_total_reserve_in_usd,
                        "mcap_fdv_fallback": mcap_fdv_fallback,
                    },
                    token_price_get_addresses_params.TokenPriceGetAddressesParams,
                ),
            ),
            cast_to=TokenPriceGetAddressesResponse,
        )


class TokenPriceResourceWithRawResponse:
    def __init__(self, token_price: TokenPriceResource) -> None:
        self._token_price = token_price

        self.get_addresses = to_raw_response_wrapper(
            token_price.get_addresses,
        )


class AsyncTokenPriceResourceWithRawResponse:
    def __init__(self, token_price: AsyncTokenPriceResource) -> None:
        self._token_price = token_price

        self.get_addresses = async_to_raw_response_wrapper(
            token_price.get_addresses,
        )


class TokenPriceResourceWithStreamingResponse:
    def __init__(self, token_price: TokenPriceResource) -> None:
        self._token_price = token_price

        self.get_addresses = to_streamed_response_wrapper(
            token_price.get_addresses,
        )


class AsyncTokenPriceResourceWithStreamingResponse:
    def __init__(self, token_price: AsyncTokenPriceResource) -> None:
        self._token_price = token_price

        self.get_addresses = async_to_streamed_response_wrapper(
            token_price.get_addresses,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["MarketChartGetParams"]


class MarketChartGetParams(TypedDict, total=False):
    days: Required[str]
    """
    data up to number of days ago You may use any integer or `max` for number of
    days
    """

    vs_currency: Required[str]
    """
    target currency of market data \\**refers to
    [`/simple/supported_vs_currencies`](/reference/simple-supported-currencies).
    """

    interval: Literal["5m", "hourly", "daily"]
    """data interval, leave empty for auto granularity"""

    precision: Literal[
        "full", "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", "16", "17", "18"
    ]
    """decimal place for currency price value"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.onchain.tokens import info_recently_updated_get_params
from ....types.onchain.tokens.info_recently_updated_get_response import InfoRecentlyUpdatedGetResponse

__all__ = ["InfoRecentlyUpdatedResource", "AsyncInfoRecentlyUpdatedResource"]


class InfoRecentlyUpdatedResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> InfoRecentlyUpdatedResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return InfoRecentlyUpdatedResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InfoRecentlyUpdatedResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return InfoRecentlyUpdatedResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        include: Literal["network"] | NotGiven = NOT_GIVEN,
        network: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InfoRecentlyUpdatedGetResponse:
        """
        This endpoint allows you to **query 100 most recently updated tokens info of a
        specific network or across all networks on GeckoTerminal**

        Args:
          include: Attributes for related resources to include, which will be returned under the
              top-level 'included' key

          network: filter tokens by provided network \\**refers to
              [/networks](/reference/networks-list)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/onchain/tokens/info_recently_updated",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "include": include,
                        "network": network,
                    },
                    info_recently_updated_get_params.InfoRecentlyUpdatedGetParams,
                ),
            ),
            cast_to=InfoRecentlyUpdatedGetResponse,
        )


class AsyncInfoRecentlyUpdatedResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncInfoRecentlyUpdatedResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInfoRecentlyUpdatedResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInfoRecentlyUpdatedResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncInfoRecentlyUpdatedResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        include: Literal["network"] | NotGiven = NOT_GIVEN,
        network: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InfoRecentlyUpdatedGetResponse:
        """
        This endpoint allows you to **query 100 most recently updated tokens info of a
        specific network or across all networks on GeckoTerminal**

        Args:
          include: Attributes for related resources to include, which will be returned under the
              top-level 'included' key

          network: filter tokens by provided network \\**refers to
              [/networks](/reference/networks-list)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/onchain/tokens/info_recently_updated",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "include": include,
                        "network": network,
                    },
                    info_recently_updated_get_params.InfoRecentlyUpdatedGetParams,
                ),
            ),
            cast_to=InfoRecentlyUpdatedGetResponse,
        )


class InfoRecentlyUpdatedResourceWithRawResponse:
    def __init__(self, info_recently_updated: InfoRecentlyUpdatedResource) -> None:
        self._info_recently_updated = info_recently_updated

        self.get = to_raw_response_wrapper(
            info_recently_updated.get,
        )


class AsyncInfoRecentlyUpdatedResourceWithRawResponse:
    def __init__(self, info_recently_updated: AsyncInfoRecentlyUpdatedResource) -> None:
        self._info_recently_updated = info_recently_updated

        self.get = async_to_raw_response_wrapper(
            info_recently_updated.get,
        )


class InfoRecentlyUpdatedResourceWithStreamingResponse:
    def __init__(self, info_recently_updated: InfoRecentlyUpdatedResource) -> None:
        self._info_recently_updated = info_recently_updated

        self.get = to_streamed_response_wrapper(
            info_recently_updated.get,
        )


class AsyncInfoRecentlyUpdatedResourceWithStreamingResponse:
    def __init__(self, info_recently_updated: AsyncInfoRecentlyUpdatedResource) -> None:
        self._info_recently_updated = info_recently_updated

        self.get = async_to_streamed_response_wrapper(
            info_recently_updated.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["NFTGetMarketsParams"]


class NFTGetMarketsParams(TypedDict, total=False):
    asset_platform_id: str
    """
    filter result by asset platform (blockchain network) \\**refers to
    [`/asset_platforms`](/reference/asset-platforms-list) filter=`nft`
    """

    order: Literal[
        "h24_volume_native_asc",
        "h24_volume_native_desc",
        "h24_volume_usd_asc",
        "h24_volume_usd_desc",
        "market_cap_usd_asc",
        "market_cap_usd_desc",
    ]
    """sort results by field Default: `market_cap_usd_desc`"""

    page: float
    """page through results Default: `1`"""

    per_page: float
    """
    total results per page Valid values: any integer between 1 and 250 Default:
    `100`
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["MarketCapChartGetResponse", "MarketCapChart"]


class MarketCapChart(BaseModel):
    market_cap: Optional[List[List[float]]] = None

    volume: Optional[List[List[float]]] = None


class MarketCapChartGetResponse(BaseModel):
    market_cap_chart: Optional[MarketCapChart] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .trade_get_params import TradeGetParams as TradeGetParams
from .info_get_response import InfoGetResponse as InfoGetResponse
from .trade_get_response import TradeGetResponse as TradeGetResponse
from .multi_get_addresses_params import MultiGetAddressesParams as MultiGetAddressesParams
from .ohlcv_get_timeframe_params import OhlcvGetTimeframeParams as OhlcvGetTimeframeParams
from .multi_get_addresses_response import MultiGetAddressesResponse as MultiGetAddressesResponse
from .ohlcv_get_timeframe_response import OhlcvGetTimeframeResponse as OhlcvGetTimeframeResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.simple import price_get_params
from ...types.simple.price_get_response import PriceGetResponse

__all__ = ["PriceResource", "AsyncPriceResource"]


class PriceResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PriceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return PriceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PriceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return PriceResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        vs_currencies: str,
        ids: str | NotGiven = NOT_GIVEN,
        include_24hr_change: bool | NotGiven = NOT_GIVEN,
        include_24hr_vol: bool | NotGiven = NOT_GIVEN,
        include_last_updated_at: bool | NotGiven = NOT_GIVEN,
        include_market_cap: bool | NotGiven = NOT_GIVEN,
        include_tokens: Literal["top", "all"] | NotGiven = NOT_GIVEN,
        names: str | NotGiven = NOT_GIVEN,
        precision: Literal[
            "full",
            "0",
            "1",
            "2",
            "3",
            "4",
            "5",
            "6",
            "7",
            "8",
            "9",
            "10",
            "11",
            "12",
            "13",
            "14",
            "15",
            "16",
            "17",
            "18",
        ]
        | NotGiven = NOT_GIVEN,
        symbols: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PriceGetResponse:
        """
        This endpoint allows you to **query the prices of one or more coins by using
        their unique Coin API IDs**

        Args:
          vs_currencies: target currency of coins, comma-separated if querying more than 1 currency.
              \\**refers to
              [`/simple/supported_vs_currencies`](/reference/simple-supported-currencies).

          ids: coins' IDs, comma-separated if querying more than 1 coin. \\**refers to
              [`/coins/list`](/reference/coins-list).

          include_24hr_change: include 24hr change, default: false

          include_24hr_vol: include 24hr volume, default: false

          include_last_updated_at: include last updated price time in UNIX, default: false

          include_market_cap: include market capitalization, default: false

          include_tokens: for `symbols` lookups, specify `all` to include all matching tokens Default
              `top` returns top-ranked tokens (by market cap or volume)

          names: coins' names, comma-separated if querying more than 1 coin.

          precision: decimal place for currency price value

          symbols: coins' symbols, comma-separated if querying more than 1 coin.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/simple/price",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "vs_currencies": vs_currencies,
                        "ids": ids,
                        "include_24hr_change": include_24hr_change,
                        "include_24hr_vol": include_24hr_vol,
                        "include_last_updated_at": include_last_updated_at,
                        "include_market_cap": include_market_cap,
                        "include_tokens": include_tokens,
                        "names": names,
                        "precision": precision,
                        "symbols": symbols,
                    },
                    price_get_params.PriceGetParams,
                ),
            ),
            cast_to=PriceGetResponse,
        )


class AsyncPriceResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPriceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPriceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPriceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncPriceResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        vs_currencies: str,
        ids: str | NotGiven = NOT_GIVEN,
        include_24hr_change: bool | NotGiven = NOT_GIVEN,
        include_24hr_vol: bool | NotGiven = NOT_GIVEN,
        include_last_updated_at: bool | NotGiven = NOT_GIVEN,
        include_market_cap: bool | NotGiven = NOT_GIVEN,
        include_tokens: Literal["top", "all"] | NotGiven = NOT_GIVEN,
        names: str | NotGiven = NOT_GIVEN,
        precision: Literal[
            "full",
            "0",
            "1",
            "2",
            "3",
            "4",
            "5",
            "6",
            "7",
            "8",
            "9",
            "10",
            "11",
            "12",
            "13",
            "14",
            "15",
            "16",
            "17",
            "18",
        ]
        | NotGiven = NOT_GIVEN,
        symbols: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PriceGetResponse:
        """
        This endpoint allows you to **query the prices of one or more coins by using
        their unique Coin API IDs**

        Args:
          vs_currencies: target currency of coins, comma-separated if querying more than 1 currency.
              \\**refers to
              [`/simple/supported_vs_currencies`](/reference/simple-supported-currencies).

          ids: coins' IDs, comma-separated if querying more than 1 coin. \\**refers to
              [`/coins/list`](/reference/coins-list).

          include_24hr_change: include 24hr change, default: false

          include_24hr_vol: include 24hr volume, default: false

          include_last_updated_at: include last updated price time in UNIX, default: false

          include_market_cap: include market capitalization, default: false

          include_tokens: for `symbols` lookups, specify `all` to include all matching tokens Default
              `top` returns top-ranked tokens (by market cap or volume)

          names: coins' names, comma-separated if querying more than 1 coin.

          precision: decimal place for currency price value

          symbols: coins' symbols, comma-separated if querying more than 1 coin.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/simple/price",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "vs_currencies": vs_currencies,
                        "ids": ids,
                        "include_24hr_change": include_24hr_change,
                        "include_24hr_vol": include_24hr_vol,
                        "include_last_updated_at": include_last_updated_at,
                        "include_market_cap": include_market_cap,
                        "include_tokens": include_tokens,
                        "names": names,
                        "precision": precision,
                        "symbols": symbols,
                    },
                    price_get_params.PriceGetParams,
                ),
            ),
            cast_to=PriceGetResponse,
        )


class PriceResourceWithRawResponse:
    def __init__(self, price: PriceResource) -> None:
        self._price = price

        self.get = to_raw_response_wrapper(
            price.get,
        )


class AsyncPriceResourceWithRawResponse:
    def __init__(self, price: AsyncPriceResource) -> None:
        self._price = price

        self.get = async_to_raw_response_wrapper(
            price.get,
        )


class PriceResourceWithStreamingResponse:
    def __init__(self, price: PriceResource) -> None:
        self._price = price

        self.get = to_streamed_response_wrapper(
            price.get,
        )


class AsyncPriceResourceWithStreamingResponse:
    def __init__(self, price: AsyncPriceResource) -> None:
        self._price = price

        self.get = async_to_streamed_response_wrapper(
            price.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from ..._models import BaseModel

__all__ = [
    "HistoryGetResponse",
    "CommunityData",
    "DeveloperData",
    "DeveloperDataCodeAdditionsDeletions4Weeks",
    "Image",
    "MarketData",
    "MarketDataCurrentPrice",
    "MarketDataMarketCap",
    "MarketDataTotalVolume",
    "PublicInterestStats",
]


class CommunityData(BaseModel):
    facebook_likes: Optional[float] = None
    """coin facebook likes"""

    reddit_accounts_active_48h: Optional[float] = None
    """coin reddit accounts active 48h"""

    reddit_average_comments_48h: Optional[float] = None
    """coin reddit average comments 48h"""

    reddit_average_posts_48h: Optional[float] = None
    """coin reddit average posts 48h"""

    reddit_subscribers: Optional[float] = None
    """coin reddit subscribers"""


class DeveloperDataCodeAdditionsDeletions4Weeks(BaseModel):
    additions: Optional[float] = None

    deletions: Optional[float] = None


class DeveloperData(BaseModel):
    closed_issues: Optional[float] = None
    """coin repository closed issues"""

    code_additions_deletions_4_weeks: Optional[DeveloperDataCodeAdditionsDeletions4Weeks] = None
    """coin code additions deletions 4 weeks"""

    commit_count_4_weeks: Optional[float] = None
    """coin commit count 4 weeks"""

    forks: Optional[float] = None
    """coin repository forks"""

    pull_request_contributors: Optional[float] = None
    """coin repository pull request contributors"""

    pull_requests_merged: Optional[float] = None
    """coin repository pull requests merged"""

    stars: Optional[float] = None
    """coin repository stars"""

    subscribers: Optional[float] = None
    """coin repository subscribers"""

    total_issues: Optional[float] = None
    """coin repository total issues"""


class Image(BaseModel):
    small: Optional[str] = None

    thumb: Optional[str] = None


class MarketDataCurrentPrice(BaseModel):
    btc: Optional[float] = None

    eur: Optional[float] = None

    usd: Optional[float] = None


class MarketDataMarketCap(BaseModel):
    btc: Optional[float] = None

    eur: Optional[float] = None

    usd: Optional[float] = None


class MarketDataTotalVolume(BaseModel):
    btc: Optional[float] = None

    eur: Optional[float] = None

    usd: Optional[float] = None


class MarketData(BaseModel):
    current_price: Optional[MarketDataCurrentPrice] = None
    """coin current price"""

    market_cap: Optional[MarketDataMarketCap] = None
    """coin market cap"""

    total_volume: Optional[MarketDataTotalVolume] = None
    """coin total volume"""


class PublicInterestStats(BaseModel):
    alexa_rank: Optional[float] = None
    """coin alexa rank"""

    bing_matches: Optional[float] = None
    """coin bing matches"""


class HistoryGetResponse(BaseModel):
    id: Optional[str] = None
    """coin ID"""

    community_data: Optional[CommunityData] = None
    """coin community data"""

    developer_data: Optional[DeveloperData] = None
    """coin developer data"""

    image: Optional[Image] = None
    """coin image url"""

    localization: Optional[Dict[str, str]] = None
    """coin localization"""

    market_data: Optional[MarketData] = None
    """coin market data"""

    name: Optional[str] = None
    """coin name"""

    public_interest_stats: Optional[PublicInterestStats] = None
    """coin public interest stats"""

    symbol: Optional[str] = None
    """coin symbol"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["ExchangeGetIDParams"]


class ExchangeGetIDParams(TypedDict, total=False):
    dex_pair_format: Literal["contract_address", "symbol"]
    """
    set to `symbol` to display DEX pair base and target as symbols, default:
    `contract_address`
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["PoolGetParams"]


class PoolGetParams(TypedDict, total=False):
    include: str
    """
    attributes to include, comma-separated if more than one to include Available
    values: `base_token`, `quote_token`, `dex`
    """

    page: int
    """page through results Default value: 1"""

    sort: Literal["h24_tx_count_desc", "h24_volume_usd_desc"]
    """sort the pools by field Default value: h24_tx_count_desc"""
